package com.jiuqi.sql.impl;

import java.util.Date;
import com.jiuqi.sql.FieldInfo;
import java.io.IOException;
import com.jiuqi.sql.DBException;
import java.io.OutputStream;
import com.jiuqi.sql.OutputRecordBuffer;

public class DBaseOutputRecordBuffer implements OutputRecordBuffer
{
    private byte[] buffer;
    private DBaseFileTail strValueList;
    
    public DBaseOutputRecordBuffer(final int recordSize, final DBaseFileTail strValueList) {
        this.buffer = new byte[recordSize];
        this.strValueList = strValueList;
    }
    
    public void writeRecord(final OutputStream stream) throws DBException {
        try {
            stream.write(32);
            stream.write(this.buffer);
        }
        catch (final IOException ex) {
            final DBException dbe = new DBException();
            throw dbe;
        }
    }
    
    public void clearBuffer() throws DBException {
    }
    
    public void writeBoolean(final FieldInfo info, final boolean value) throws DBException {
        RW.writeBoolean(this.buffer, info.offset, value);
    }
    
    public void writeDouble(final FieldInfo info, final double value) throws DBException {
        RW.writeDouble(this.buffer, info.offset, value);
    }
    
    public void writeInt(final FieldInfo info, final int value) throws DBException {
        RW.writeInt(this.buffer, info.offset, value);
    }
    
    public void writeString(final FieldInfo info, final String value) throws DBException {
        if (value == null) {
            this.writeNull(info);
        }
        else {
            RW.writeInt(this.buffer, info.offset, this.strValueList.size());
            this.strValueList.add(value);
        }
    }
    
    public void writeLong(final FieldInfo info, final long value) throws DBException {
        RW.writeLong(this.buffer, info.offset, value);
    }
    
    public void writeDate(final FieldInfo info, final Date value) throws DBException {
        if (value == null) {
            this.writeNull(info);
        }
        else {
            RW.writeDateTime(this.buffer, info.offset, value);
        }
    }
    
    public void writeNull(final FieldInfo info) throws DBException {
        switch (info.type) {
            case 1: {
                this.buffer[info.offset] = 32;
                break;
            }
            case 3: {
                for (int i = 0; i < 8; ++i) {
                    this.buffer[info.offset + i] = 0;
                }
                break;
            }
            case 5: {
                for (int i = 0; i < 4; ++i) {
                    this.buffer[info.offset + i] = 0;
                }
                break;
            }
            case 6: {
                for (int i = 0; i < 4; ++i) {
                    this.buffer[info.offset + i] = -1;
                }
                break;
            }
            case 2:
            case 8: {
                for (int i = 0; i < 8; ++i) {
                    this.buffer[info.offset + i] = 32;
                }
                break;
            }
        }
    }
}
